@@ -31,6 +31,7 @@
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include <libgnomevfs/gnome-vfs-mime-info-cache.h>
 #include <libxml/parser.h>
+#include <libxml/xmlreader.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <fcntl.h>
@@ -41,6 +42,8 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 
+#define DESKTOP_ENTRY_GROUP                "Desktop Entry"
+
 static gboolean
 recursive_mkdir (const char *path)
 {
@@ -229,7 +232,7 @@ eel_mime_add_application (const char *mime_type,
 	application_dir = get_user_dir ("applications");
 
 	/* Get a filename */
-	basename = g_strdup_printf ("%s.desktop", name);
+	basename = g_strdup_printf ("%s-usercreated.desktop", name);
 	filename = g_build_filename (application_dir, basename, NULL);
 
 	i = 1;
@@ -237,7 +240,7 @@ eel_mime_add_application (const char *mime_type,
 		g_free (basename);
 		g_free (filename);
 		
-		basename = g_strdup_printf ("%s%d.desktop", name, i);
+		basename = g_strdup_printf ("%s-usercreated-%d.desktop", name, i);
 		filename = g_build_filename (application_dir, basename, NULL);
 		
 		i++;
@@ -270,6 +273,87 @@ eel_mime_add_application (const char *mime_type,
 	return ret;
 }
 
+GnomeVFSMimeApplication *
+eel_mime_add_custom_mime_type_for_desktop_file (const char *desktop_file,
+						const char *mime_type)
+{
+	GnomeVFSMimeApplication *ret;
+	GKeyFile *keyfile;
+	char *application_dir;
+	char *filename;
+	char *basename, *new_basename;
+	char *extension;
+	int i;
+
+	if (desktop_file == NULL) {
+		return NULL;
+	}
+
+	if (mime_type == NULL) {
+		return NULL;
+	}
+	
+	if (!ensure_application_dir ()) {
+		return NULL;
+	}
+
+	application_dir = get_user_dir ("applications");
+
+	/* Get a filename */
+	basename = g_path_get_basename (desktop_file);
+	/* Strip off extension */
+	extension = strrchr (basename, '.');
+	if (extension != NULL) {
+		*extension = 0;
+	}
+
+	new_basename = g_strdup_printf ("%s-usercustom.desktop", basename);
+	filename = g_build_filename (application_dir, new_basename, NULL);
+
+	i = 1;
+	while (g_file_test (filename, G_FILE_TEST_EXISTS)) {
+		g_free (new_basename);
+		g_free (filename);
+		new_basename = g_strdup_printf ("%s-usercustom-%d.desktop", basename, i++);
+		filename = g_build_filename (application_dir, new_basename, NULL);
+	}
+
+	g_free (basename);
+	
+	keyfile = g_key_file_new ();
+	if (g_key_file_load_from_file (keyfile, desktop_file,
+				       G_KEY_FILE_KEEP_COMMENTS |
+				       G_KEY_FILE_KEEP_TRANSLATIONS, NULL)) {
+		char *desktop_text;
+		char *new_mimetypes;
+
+		new_mimetypes = g_strconcat (mime_type, ";", NULL);
+		g_key_file_set_string (keyfile, DESKTOP_ENTRY_GROUP, "MimeType", new_mimetypes);
+		g_free (new_mimetypes);
+		g_key_file_set_boolean (keyfile, DESKTOP_ENTRY_GROUP, "NoDisplay", TRUE);
+		desktop_text = g_key_file_to_data (keyfile, NULL, NULL);
+
+		if (write_desktop_file (filename, desktop_text)) {
+			ret = gnome_vfs_mime_application_new_from_desktop_id (new_basename);
+			run_update_command ("update-desktop-database", "applications");
+		} else {
+			ret = NULL;
+		}
+        	g_free (desktop_text);
+	} else {
+		ret = NULL;
+	}
+	
+	g_key_file_free (keyfile);
+	g_free (new_basename);
+	g_free (filename);
+	g_free (application_dir);
+	
+	return ret;
+}
+
+
+
 static char *
 get_override_filename (void)
 {
@@ -501,6 +585,7 @@ open_temp_cache_file (const char *dir, char **filename, GError **error)
               g_set_error (error, G_FILE_ERROR,
                            g_file_error_from_errno (errno),
                            "%s", g_strerror (errno));
+	      *filename = NULL;
               break;
             }
         }
@@ -697,6 +782,39 @@ arg_is_exec_param (const char *param)
 	return FALSE;
 }
 
+GnomeVFSMimeApplication *
+eel_mime_check_for_desktop_duplicates (const char *mime_type,
+				       const char *desktop_file)
+{
+	GList *applications;
+	GList *list;
+
+	if (mime_type == NULL) {
+		return NULL;
+	}
+	
+	applications = gnome_vfs_mime_get_all_applications (mime_type);
+	if (applications == NULL) {
+		return NULL;
+	}
+
+	for (list = applications; list; list = list->next) {
+		GnomeVFSMimeApplication *application;
+		const char *path;
+		
+		application = list->data;
+		path = gnome_vfs_mime_application_get_desktop_file_path (application);
+		if (strcmp (desktop_file, path) == 0) {
+			g_list_free (applications);
+			return gnome_vfs_mime_application_copy (application);
+		}
+	}
+	g_list_free (applications);
+
+	return NULL;
+}
+
+
 GnomeVFSMimeApplication *
 eel_mime_check_for_duplicates (const char *mime_type,
 			       const char *command_line)
@@ -784,3 +902,264 @@ eel_mime_check_for_duplicates (const char *mime_type,
 
 	return NULL;
 }
+
+/* this gives us a number of the language in the current language list,
+   the lower the number the "better" the translation */
+static int
+language_level (const char *lang)
+{
+	const char * const *lang_list;
+	int i;
+
+	if (lang == NULL)
+		lang = "C";
+
+	/* The returned list is sorted from most desirable to least
+            desirable and always contains the default locale "C". */
+	lang_list = g_get_language_names();
+
+	for (i = 0; lang_list[i]; i++) {
+		if (strcmp (lang_list[i], lang) == 0)
+			return i;
+	}
+
+	return -1;
+}
+
+static int
+read_next (xmlTextReaderPtr reader) 
+{
+	int depth;
+	int ret;
+	
+	depth = xmlTextReaderDepth (reader);
+	
+	ret = xmlTextReaderRead (reader);
+	while (ret == 1) {
+		if (xmlTextReaderDepth (reader) == depth) {
+			return 1;
+		} else if (xmlTextReaderDepth (reader) < depth) {
+			return 0;
+		}
+		ret = xmlTextReaderRead (reader);
+	}
+
+	return ret;
+}
+
+static char *
+handle_simple_string (xmlTextReaderPtr reader, int *_ret)
+{
+	int ret;
+	char *text = NULL;
+
+	ret = xmlTextReaderRead (reader);
+	while (ret == 1) {
+		xmlReaderTypes type;		
+		type = xmlTextReaderNodeType (reader);
+		if (type == XML_READER_TYPE_TEXT) {
+			if (text != NULL) {
+				g_free (text);
+			}
+			text = g_strdup ((char *)xmlTextReaderConstValue (reader));
+		}
+
+		ret = read_next (reader);
+	}
+	*_ret = ret;
+	return text;
+}
+
+static void
+handle_mime_type (EelMimeTypeInfo *info, xmlTextReaderPtr reader)
+{
+	xmlReaderTypes type;
+	const char *name;
+	int depth;
+	int previous_lang_level = INT_MAX;
+	char *comment;
+	gboolean read;
+	int ret;
+	
+	depth = xmlTextReaderDepth (reader);
+	
+	ret = xmlTextReaderRead (reader);
+
+	if (xmlTextReaderDepth (reader) != depth + 1) {
+		return;
+	}
+	
+	while (ret == 1) {
+		read = FALSE;
+		type = xmlTextReaderNodeType (reader);
+		
+		if (type == XML_READER_TYPE_ELEMENT) {
+			name = (char *)xmlTextReaderConstName (reader);
+			if (!strcmp (name, "comment")) {
+				const char *lang;
+				int lang_level;
+				
+				lang = (char *)xmlTextReaderConstXmlLang (reader);
+				
+				lang_level = language_level (lang);
+				
+				if (lang_level != -1 &&
+				    lang_level < previous_lang_level) {
+					comment = handle_simple_string (reader, &ret);
+					g_free (info->description);
+					info->description = comment;
+					previous_lang_level = lang_level;
+					read = TRUE;
+				}
+			} 
+		}
+		if (!read) {
+			ret = read_next (reader);
+		} else {
+			ret = 1;
+		}
+	}
+
+	if (info->description == NULL) {
+		info->description = g_strdup (info->mime_type);
+	}
+}
+
+static GList *
+parse_mime_xml (const char *filename)
+{
+	xmlTextReaderPtr reader;
+	int ret;
+	GList *list;
+	char *mimetype;
+	EelMimeTypeInfo *info;
+	const char *name;
+	xmlChar *xml_text = NULL;
+	gboolean read;
+
+	reader = xmlNewTextReaderFilename (filename);
+
+	if (reader == NULL) {
+		return NULL;
+	}
+
+	list = NULL;
+	
+	ret = xmlTextReaderRead (reader);
+	while (ret == 1) {
+		read = FALSE;
+		if (xmlTextReaderNodeType (reader) == XML_READER_TYPE_ELEMENT &&
+		    xmlTextReaderDepth (reader) == 1) {
+			name = xmlTextReaderConstName(reader);
+			if (name != NULL && strcmp (name, "mime-type") == 0) {
+				mimetype = NULL;
+				xml_text = xmlTextReaderGetAttribute (reader, (guchar *)"type");
+				if (xml_text != NULL) {
+					mimetype = g_strdup ((char *)xml_text);
+					xmlFree (xml_text);
+				}
+
+				if (mimetype != NULL) {
+					info = g_new0 (EelMimeTypeInfo, 1);
+					info->mime_type = mimetype;
+					handle_mime_type (info, reader);
+					list = g_list_prepend (list, info);
+					read = TRUE;
+				}
+			}
+		}
+		if (read) {
+			ret = 1;
+		} else {
+			ret = xmlTextReaderRead (reader);
+		}
+	}
+
+	xmlFreeTextReader(reader);	
+	return g_list_reverse (list);
+}
+
+static GList *
+visit_mime_dir (const char *base_dir)
+{
+	char *dir;
+	GDir *gdir;
+	const char *file;
+	char *full_path;
+	GList *list, *ret;
+
+	list = NULL;
+	dir = g_build_filename (base_dir, "mime/packages", NULL);
+	
+	gdir = g_dir_open (dir, 0, NULL);
+	if (gdir) {
+		while ((file = g_dir_read_name (gdir)) != NULL) {
+			full_path = g_build_filename (dir, file, NULL);
+			ret = parse_mime_xml (full_path);
+			g_free (full_path);
+			list = g_list_concat (list, ret);
+		}
+
+		g_dir_close (gdir);
+	}
+
+	g_free (dir);
+	
+	return list;
+}
+
+static gint
+compare_mime_info (gconstpointer  a,
+		   gconstpointer  b)
+{
+	const EelMimeTypeInfo *aa, *bb;
+
+	aa = a;
+	bb = b;
+	return strcmp (aa->mime_type, bb->mime_type);
+}
+
+
+GList *
+eel_mime_get_availible_mime_types (void)
+{
+	const char * const *dirs;
+	GList *res, *l, *next;
+	char *last_mime;
+	EelMimeTypeInfo *info;
+
+	res = visit_mime_dir (g_get_user_data_dir ());
+	
+	for (dirs = g_get_system_data_dirs (); 
+	     *dirs; 
+	     dirs++) {
+		l = visit_mime_dir (*dirs);
+		res = g_list_concat (res, l);
+	}
+
+	res = g_list_sort (res, compare_mime_info);
+
+	last_mime = NULL;
+	for (l = res; l != NULL; l = next) {
+		info = l->data;
+		next = l->next;
+
+		if (last_mime && strcmp (info->mime_type, last_mime) == 0) {
+			eel_mime_type_info_free (info);
+			res = g_list_delete_link (res, l);
+		} else {
+			last_mime = info->mime_type;
+		}
+	}
+	
+	return res;
+}
+
+void
+eel_mime_type_info_free (EelMimeTypeInfo *info)
+{
+	g_free (info->mime_type);
+	g_free (info->description);
+	g_free (info);
+}
+
